@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Red Hat, Inc.
+ * Copyright (C) 2011,2016 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -536,6 +536,30 @@ reprocess:
 				format++;
 			}
 			goto reprocess;
+		case 'z':
+			format++;
+			if (sizeof(size_t) == sizeof(long long)) {
+				type_longlong = QB_TRUE;
+			} else {
+				type_long = QB_TRUE;
+			}
+			goto reprocess;
+		case 't':
+			format++;
+			if (sizeof(ptrdiff_t) == sizeof(long long)) {
+				type_longlong = QB_TRUE;
+			} else {
+				type_long = QB_TRUE;
+			}
+			goto reprocess;
+		case 'j':
+			format++;
+			if (sizeof(intmax_t) == sizeof(long long)) {
+				type_longlong = QB_TRUE;
+			} else {
+				type_long = QB_TRUE;
+			}
+			goto reprocess;
 		case 'd': /* int argument */
 		case 'i': /* int argument */
 		case 'o': /* unsigned int argument */
@@ -604,9 +628,10 @@ reprocess:
 			int arg_int;
 			unsigned char arg_char;
 
-			if (location + sizeof (unsigned int) > max_len) {
+			if (location + sizeof (unsigned char) > max_len) {
 				return max_len;
 			}
+			/* va_arg only takes fully promoted types */
 			arg_int = va_arg(ap, unsigned int);
 			arg_char = (unsigned char)arg_int;
 			memcpy (&serialize[location], &arg_char, sizeof (unsigned char));
@@ -737,6 +762,35 @@ reprocess:
 				type_longlong = QB_TRUE;
 			}
 			goto reprocess;
+		case 'z':
+			fmt[fmt_pos++] = *format;
+			format++;
+			if (sizeof(size_t) == sizeof(long long)) {
+				type_long = QB_FALSE;
+				type_longlong = QB_TRUE;
+			} else {
+				type_longlong = QB_FALSE;
+				type_long = QB_TRUE;
+			}
+			goto reprocess;
+		case 't':
+			fmt[fmt_pos++] = *format;
+			format++;
+			if (sizeof(ptrdiff_t) == sizeof(long long)) {
+				type_longlong = QB_TRUE;
+			} else {
+				type_long = QB_TRUE;
+			}
+			goto reprocess;
+		case 'j':
+			fmt[fmt_pos++] = *format;
+			format++;
+			if (sizeof(intmax_t) == sizeof(long long)) {
+				type_longlong = QB_TRUE;
+			} else {
+				type_long = QB_TRUE;
+			}
+			goto reprocess;
 		case 'd': /* int argument */
 		case 'i': /* int argument */
 		case 'o': /* unsigned int argument */
